;;
;;  armada
;;
;;  Copyright (C) 2014 Daniel Verasson
;; 
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;  
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;;  GNU General Public License for more details.
;;  
;;  You should have received a copy of the GNU General Public License
;;  along with this program. If not, see <http://www.gnu.org/licenses/>.
;;
;;
;;
;; keywords for fomus........................................................................................................................
(define :staff ':staff)
(define :voice ':voice)
(define :dynamics ':dynamics)
(define :articulation ':articulation)
(define :format ':format)
(define :measure ':measure)
(define :tempo ':tempo)
;; keywords for csound.......................................................................................................................
(define :instr ':instr)
(define :include ':include)
;; AUSGELAGERTE LETS
(define 1st (lambda (ls) (list-ref ls 0)))
(define 2nd (lambda (ls) (list-ref ls 1)))
(define 3rd (lambda (ls) (list-ref ls 2)))
(define 4th (lambda (ls) (list-ref ls 3)))
(define 5th (lambda (ls) (list-ref ls 4)))
(define show (lambda a (for-each display a) (newline)))
(define show-n (lambda a (for-each display a)))
(define member? (lambda (el ls) (not (equal? (member el ls) #f))))
(define optionals '(:staff :voice :dynamics :articulation :format :measure :tempo))
(define count-opts (lambda (ls) (letrec ((h (lambda (l1 l2 n)
					      (cond ((null? l1) n)
						    ((member? (car l1) l2) (h (cdr l1) l2 n))
						    ((member? (car l1) optionals) (h (cdr l1) (cons (car l1) l2) (+ n 1)))
						    (else (h (cdr l1) l2 n))))))
				  (h ls '() 0))))
(define rotate (lambda (l) (append (cdr l) (list (car l)))))
(define repn (lambda (ls n) (letrec ((h (lambda (l res z)
					  (cond ((= z n) res)
						(else (h (rotate l) (append res (list (car l))) (+ z 1)))))))
			      (h ls '() 0))))
(define any-optionals? (lambda (ls) (letrec ((h (lambda (l)
						  (cond ((null? l) #f)
							((member? (car l) optionals) #t)
							(else (h (cdr l)))))))
				      (h ls))))
(define show+n (lambda a (for-each display a) (newline) (newline)))
(define find-opt-csound (lambda (ls) (cond ((and (member? :instr ls) (member? :include ls)) 4)
					   ((or (member? :instr ls) (member? :include ls)) 2)
					   (else 0))))
(define list-tester-for-el (lambda (x) (if (not (list? x)) (list x) x)))
(define list-tester-for-lists (lambda (x) (if (and (not (null? x))
						   (not (list? (car x)))) (list x) x)))
(define make-bracket (lambda (x) (string-append "[" x "]")))
(define find-opt (lambda (list opt default) (let ((k (memq opt list))) (if k (cadr k) default))))
(define scale-dynamics (lambda (n)
			 (let ((m (cond ((<= n 1) 0.05) ((>= n 8) 8.0) (else (exact->inexact (round n))))))
			   (case m ((0.05) 0.05) ((1.0) 0.1) ((2.0) 0.2) ((3.0) 0.3) ((4.0) 0.5) ((5.0) 0.6) ((6.0) 0.8) ((7.0) 0.9) ((8.0) 1.0)))))
(define show-staffs (lambda (n) (letrec ((h (lambda (z)
					      (cond ((> z n) (newline))
						    (else (show "part <id " z " , inst default> ") (h (+ z 1)))))))
				  (h 1))))
(define empty-string? (lambda (string) (letrec ((h (lambda (s res)
						     (cond ((= (string-length s) 0) res)
							   ((equal? (string-ref s 0) #\space)
							    (h (substring s 1 (string-length s)) '#t))
							   (else (h (substring s 1 (string-length s)) '#f))))))
					 (h string '#t))))
(define prepare (lambda (p proc) (if (empty-string? (car p)) "" (proc (car p)))))
(define erase-sco-opts (lambda (ls)
			 (letrec ((h (lambda (l res)
				       (cond ((null? l)
					      res)
					     ((or (equal? (car l) :instr)
						  (equal? (car l) :include))
					      (h (cddr l) res))
					     (else (h (cdr l) (append res (list (car l)))))))))
			   (h ls '()))))
(define display-car (lambda (l) (if (string? (car l)) (show-n "\"" (car l) "\" ") (show-n (car l) " "))))
(define display-all-cars (lambda (l) (for-each display-car l) (newline)))
(define make-fomus-file (lambda (args quantity- start- duration- pitch- staff- voice- dynamics- articulation- measure- tempo- init)
			  (letrec ((worker (lambda (quantity- start- duration- pitch- staff- voice- dynamics- articulation-)
					     (if (> quantity- 0)
						 (begin
						   (show "part " (car staff-))
						   (show "time " (car start-)
							 " dur " (car duration-)
							 " voice " (car voice-)
							 " pitch " (car pitch-) " "
							 (prepare articulation- make-bracket)
							 " dyn " (car dynamics-)
							 " ; ")
						   (worker 
						    (- quantity- 1)
						    (rotate start-)
						    (rotate duration-)
						    (rotate pitch-)
						    (rotate staff-)
						    (rotate voice-)
						    (rotate dynamics-)
						    (rotate articulation-)))))))
			    (let ((number-of-staffs (apply max staff-)))
			      (show+n "// generated by \"armada\" - (C) 2014 daniel verasson")
			      (show "quartertones = yes")
			      (if (member? :dynamics args) (show "dyns = yes"))
			      (show
			       "inst <id default, template default, staves "
			       "(<clefs (<instclef treble , octs-down 0 , octs-up 0 , ledgers-down 0 > "
			       "<instclef bass , octs-up 0 , octs-down 0 ledgers-up 8, clef-preference 1/2 >)>)>")
			      (show-staffs number-of-staffs)
			      (for-each (lambda (t)
					  (show "part 1 ti "(1st t)" dur 0 mark [tempo \"*"
						(inexact->exact (2nd t))
						"* = #\" "(3rd t)"] ;")) tempo-)
			      (for-each (lambda (m)
					  (show
					   "time " (1st m) " |timesig ("(2nd m)" "(3rd m)")|")) measure-)
			      (worker quantity- start- duration- pitch- staff- voice- dynamics- articulation-)))))
(define make-csound-score (lambda (args-C)
			    (let* ((filepath (1st args-C))
				   (number-of-lines (2nd args-C))
				   (instr (map inexact->exact (list-tester-for-el (find-opt args-C :instr 1))))
				   (tables (find-opt args-C :include ""))
				   (p-fields (map list-tester-for-el (erase-sco-opts (cddr args-C)))))
			      (letrec ((h (lambda (n l instr- init)
					    (if (> n 0)
						(cond ((= init 0)
						       (cond ((empty-string? tables)
							      (h n l instr- 1))
							     (else (show+n "#include \""tables"\"")
								   (h n l instr- 1))))
						      (else (show-n "i" (car instr-) " ")
							    (display-all-cars l)
							    (h (- n 1) (map rotate l) (rotate instr-) 1)))))))
				(with-output-to-file filepath (lambda () (begin (show+n ";; generated by \"armada\" - (C) 2014 daniel verasson")
										(h number-of-lines p-fields instr 0))))))))
(define armada
  (lambda args
    (let* ((filename (1st args))
	   (suffix (substring filename (- (string-length filename) 3) (string-length filename)))
	   (filename-without-suffix (substring filename 0 (- (string-length filename) 4)))
	   (quantity (2nd args))
	   (show-duration (let* ((arg3 (if (pair? (3rd args)) (3rd args) (list (3rd args))))
				 (arg4 (if (pair? (4th args)) (4th args) (list (4th args))))
				 (s (repn arg3 (2nd args)))
				 (d (repn arg4 (2nd args)))
				 (start+duration (map + s d)))
			    (apply max start+duration))))
      (cond ((or (and (equal? suffix "sco") (< (length args) 4))
		 (and (equal? suffix "fms") (< (length args) 5)))
	     (error "insufficient arguments"))
	    ((equal? (car (reverse args)) #f)
	     (begin (show-n "duration of \"" filename "\": " show-duration)
		    (if (equal? suffix "fms") (show " quarter-notes") (show " seconds"))))
	    ((or (and (equal? suffix "sco") (any-optionals? args))
		 (and (equal? suffix "fms") (or (> (length args) (+ 5 (* (count-opts args) 2)))
						(> (length args) (- (length args) (find-opt-csound args))))))
	     (error "too many useless arguments"))
	    (else
	     (newline)
	     (show+n "START foc")
	     (show+n "writing \""filename"\"")
	     (cond ((equal? suffix "fms")
		    (let* ((start (list-tester-for-el (3rd args)))
			   (duration (list-tester-for-el (4th args)))
			   (pitch (list-tester-for-el (5th args)))
			   (staff (map inexact->exact (list-tester-for-el (find-opt args :staff 1))))
			   (voice (list-tester-for-el (find-opt args :voice 1)))
			   (dynamics (map scale-dynamics (list-tester-for-el (find-opt args :dynamics 5))))
			   (articulation (list-tester-for-el (find-opt args :articulation "")))
			   (format (list-tester-for-el (find-opt args :format "ly")))
			   (measure (list-tester-for-lists (find-opt args :measure '((0 4 4)))))
			   (tempo (list-tester-for-lists (find-opt args :tempo '())))
			   (run-fomus (lambda (x) (system (string-append "fomus -o " filename-without-suffix x " " filename)))))
		      (with-output-to-file (string-append filename-without-suffix".fms")
			(lambda () (make-fomus-file args quantity start duration pitch staff voice dynamics articulation measure tempo 0)))
		      (for-each (lambda (s) (run-fomus (string-append "." s))) format)))
		   ((equal? suffix "sco")
		    (make-csound-score args))
		   (else (error "wrong suffix")))
	     (show "foc FINISHED"))))))