;; rundet auf x nachkommastellen

(define (roundx n nachkommastellen)
  (/ (round (* n (expt 10.0 nachkommastellen))) (expt 10.0 nachkommastellen)))

;; (roundx 1.1234 2)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; zieht die x-te wurzel aus n

(define (root n x)
  (expt n (/ 1 x)))

;; (root 27 3)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; berechnet alle ganzzahligen teiler einer zahl n

(define (divs n)
  (letrec ((helper
	    (lambda (result zaehler)
	      (cond ((> zaehler (/ n 2))
		     (reverse (cons n result)))
		    ((integer? (/ n zaehler))
		     (helper (cons zaehler result) (+ zaehler 1)))
		    (else (helper result (+ zaehler 1)))))))
    (helper '() 1)))

;; (divs 123456)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; erstellt eine liste aus einer bestimmten anzahl an werten ab einem startwert und mit festen abstand

(define (dist-helper proc)
  (lambda (start abstand anzahl)
    (letrec ((h (lambda (n neuer-wert l)
		  (if (= n anzahl)
		      (reverse l)
		      (h (+ n 1) (proc neuer-wert abstand) (cons (round4 neuer-wert) l))))))
      (h 0 start '()))))

(define dist (dist-helper +))

(define dist* (dist-helper *))

;; (dist 1 3 5)
;; (dist* 1 10 3)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;;  interpolation (linear/proportional)

(define (interpolate proc1 proc2 proc3)
  (lambda (start ende anzahl)
    (if (< anzahl 2) (error "anzahl muss mind. 2 sein!")
	(let ((i (proc1 (proc2 ende start) (- anzahl 1))))
	  (proc3 start i anzahl)))))

(define lini (interpolate / - dist))

(define propi (interpolate root / dist*))

;; (lini 0 100 11)
;; (propi 10 10000 4)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; berechnet die fibonacci-zahlen in einer bestimmten anzahl von einem startwert an

(define (fibonacci start anzahl)
  (letrec ((h (lambda (letzter vorletzter zaehler1 zaehler2 liste)
		(cond ((> zaehler2 anzahl)
		       (reverse liste))
		      ((= start 1)
		       (cond ((= anzahl 1) '(0))
			     ((= anzahl 2) '(0 1))
			     (else (h (+ vorletzter letzter) letzter zaehler1 (+ zaehler2 1) (cons vorletzter liste)))))
		      ((> start 1)
		       (if (= zaehler1 start)
			   (h (+ vorletzter letzter) letzter zaehler1 (+ zaehler2 1) (cons letzter liste))
			   (h (+ vorletzter letzter) letzter (+ zaehler1 1) zaehler2 liste)))))))
    (if (or (< start 1)
	    (< anzahl 1))
	(error "zu kleiner Wert"))
    (h 1 0 2 1 '())))

;; (fibonacci 1 8)     -> (0 1 1 2 3 5 8 13)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; tendenzmaske mit einem integral, beschrieben durch die funktionen f1 und f2 auf dem intervall [x1, x2]

(define (integral-tend f1 f2 x1 x2 anzahl)
  (let ((function1 (function x1 x2 anzahl f1))
	(function2 (function x1 x2 anzahl f2)))
    (map (lambda (x y) (round3 (+ y (random (- x y))))) function1 function2)))

;; (integral-tend (lambda (x) 3) (lambda (x) -3) 0 1 10)
;; (integral-tend (lambda (x) 3.0) (lambda (x) -3.0) 0 1 10)
;; (integral-tend (lambda (x) (sin-phase x)) (lambda (x) 0) 0 360 100)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; tendenzmaske ueber 2 linearen funktionen

(define (tend unten-start unten-ende oben-start oben-ende anzahl)
  (integral-tend (lambda (x) (+ (* (- unten-ende unten-start) x) unten-start))
		 (lambda (x) (+ (* (- oben-ende oben-start) x) oben-start)) 0 1 anzahl))

;; (tend 0 1 0 1 100)
;; (tend 5 5 10 0 40)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; erzeugt eine liste mit einer bestimmten anzahl an beliebigen zahlen zwischen 2 werten

(define (rand minimum maximum anzahl)
  (letrec ((helper (lambda (n l)
		     (if (= n anzahl)
			 l
			 (helper (+ n 1) (cons (round3 (+ minimum (/ (random (* (expt 10.0 3) (- maximum minimum))) (expt 10 3)))) l))))))
    (helper 0 '())))

;; (rand 0 20 5)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; sublist

(define (sublist liste start ende)
  (if (null? liste) (error "liste zu klein"))
  (letrec ((h (lambda (l s anz)
		(cond ((= anz 0) '())
		      ((= s 0) (cons (car l) (h (cdr l) 0 (- anz 1))))
		      (else (h (cdr l) (- s 1) anz))))))
    (h liste start (+ 1 (- ende start)))))

;; (sublist '(a b c d e) 0 1)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; addiert/subtrahiert/dividiert/multipliziert jedes element der liste mit einem bestimmten wert

(define (procmap liste proc . wert)
  (map (lambda (n) (apply proc n wert)) liste))

;; (procmap '(1 2 3) + 2)
;; (procmap '(1 2 3) + 2 3 4 5)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; wiederholt jedes element der liste n-mal

(define (rep-each liste n)
  (rep-rand liste n n))

;; (rep-each '(1 2 3) 2)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; wiederholt jedes element der liste zwischen in der anzahl zwischen n-min und n-max

(define (rep-rand liste n-min n-max)
  (letrec ((h (lambda (l res)
		(if (null? l)
		    res
		    (h (cdr l) (append res (rep (car l) (+ n-min (random (- (+ n-max 1) n-min))))))))))
    (h liste '())))

;; (rep-rand '(1 2 3) 2 4)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; ermittelt das arithmetische mittel einer liste

(define (mean liste)
  (exact->inexact (/ (apply + liste) (length liste))))

;; (mean '(0 1 2 10))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; quicksort

(define (sort liste)
  (letrec ((helper (lambda (a b liste z)
		     (cond ((null? a) (cons liste (cons b (cons z '()))))
			   (else
			    (let ((x (car a)))
			      (if (<= x b) 
				  (helper (cdr a) b (cons x liste) z)
				  (helper (cdr a) b liste (cons x z))))))))
	   (partition (lambda (liste)
			(helper (cdr liste) (car liste) '() '()))))
    (cond ((null? liste) liste)
	  (else
	   (let ((l-part (partition liste)))
	     (append (sort (car l-part)) (cons (cadr l-part) (sort (caddr l-part)))))))))

;; (sort (rand 0 10 100))

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; mischt mehrere listen in einer bergeordneten liste miteinander

(define (mix listen-liste anzahl) ; format: '((1 4 7) (2 5 8) (3 6 9))) -> (1 2 3 4 5 6 7 8 9)
  (letrec ((h (lambda (l res)
		(let ((lr (length res)))
		  (cond ((= lr anzahl)
			 res)
			((> lr anzahl)
			 (sublist res 0 (- anzahl 1)))
			(else (h (map rotate l) (append res (map car l)))))))))
    (h listen-liste '())))

;; (mix '((1 4 7) (2 5 8) (3 6 9)) 9)
;; (mix '((1 2 3) (a b) ("q")) 9)

;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; aendert ein element in einer liste

(define (replace liste index . element)
  (if (null? element)
      (append (sublist liste 0 (- index 1)) (sublist liste (+ index 1) (- (length liste) 1)))
      (append (sublist liste 0 (- index 1)) element (sublist liste (+ index 1) (- (length liste) 1)))))

;; (replace '(0 1 2) 0 99)
;; (replace '(0 1 2) 2 99 123)
;; (replace '(0 1 2) 1)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;; wiederholt eine liste n-mal

(define (rep-list liste n)
  (apply append (rep liste n)))

;; (rep-list '(1 2 3) 2)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;; einsatzabstaende werden zu absoluten zeitangaben 

(define (entry start-zeitpunkt liste)
  (letrec ((h (lambda (l result)
		(if (null? l)
		    (reverse result)
		    (h (cdr l) (cons (+ (car l) (car result)) result))))))
    (h liste (cons start-zeitpunkt '()))))

;; (entry 10 '(1 2 3 4)) -> (0 1 3 6 10)
;; (entry 0 (rep-list '(1 1 0.5 0.5) 2))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;; member? 
(define (member-helper element liste)
  (if (member element liste) #t #f))

(define (member? element liste)
  (if (integer? element)
      (or (member-helper (inex element) liste)
	  (member-helper (round0 element) liste))
      (member-helper element liste)))

;; (member? 3 '(1 a 2 3.0))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;; list-ref mit einer index-liste, anstelle eines einzelnen index

(define (index liste indexliste)
  (map (lambda (index) (list-ref liste index)) (map round0 indexliste)))

;; (index '(10 20 30) '(0 1 0 2))
;; (index '(10 20 30) '(0.3 1.6))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

(define (list-proc-helper proc)
  (lambda (liste1 liste2 anzahl)
    (letrec ((h (lambda (l1 l2 result zaehler)
		  (if (= zaehler anzahl)
		      result
		      (h (append (cdr l1) (list (car l1))) (append (cdr l2) (list (car l2))) (append result (list (proc (car l1) (car l2)))) (+ zaehler 1))))))
      (h liste1 liste2 '() 0))))

;; addiert die werte zweier listen miteinander

(define list-add (list-proc-helper +))

;; (list-add '(10 20) '(1 2 3) 5)

;; multipliziert die werte zweier listen miteinander

(define list-mult (list-proc-helper *))

;; (list-mult '(10 20) '(1 2 3) 5)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; roundx...

(define (round0 n)
  (ex (round n)))
(define (round-helper j)
  (lambda (n) (roundx n j)))
(define round1 (round-helper 1.0))
(define round2 (round-helper 2.0))
(define round3 (round-helper 3.0))
(define round4 (round-helper 4.0))
(define round5 (round-helper 5.0))
(define round6 (round-helper 6.0))
(define round7 (round-helper 7.0))
(define round8 (round-helper 8.0))
(define round9 (round-helper 9.0))
(define round10 (round-helper 10.0))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; exact->inexact

(define (inex n)
  (exact->inexact n))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; inexact->exact

(define (ex n)
  (inexact->exact n))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; logarithmus von x ueber gegebener basis

(define (logg x basis)
  (/ (log x) (log basis)))

;; (logg 8 2)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; erzeugt die n-te generation zu einer startzelle nach gegebenen regeln
;; startzelle: '(1 2)
;; regeln:     '((1 2) (2 1 2)) -> aus 1 wird 2 / aus 2 wird 1 2

(define (cells startzelle regeln generation)
  (let ((n-gen (lambda (el ll)
		 (letrec ((h2 (lambda (ll)
				(if (= (caar ll) el)
				    (cdar ll)
				    (h2 (cdr ll))))))
		   (h2 ll)))))
    (letrec ((h (lambda (neu alt gen)
		  (cond ((= gen generation)
			 alt)
			((null? alt)
			 (h '() neu (+ gen 1)))
			(else (h (append neu (n-gen (car alt) regeln)) (cdr alt) gen))))))
      (h '() startzelle 0))))

;; (cells '(1) '((1 2) (2 1 2)) 4)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; faktoren fuer mikrotoene

(define (mikro cent)
  (expt 2 (/ (/ cent 100) 12.)))

;; (mikro 100)        -> 1.059...     //   1  halbton      aufwaerts
;; (mikro 1200)       -> 2            //   1  oktave       aufwaerts
;; (mikro (/ -200 3)) -> 0.962...     //   1  drittelton   abwaerts


;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; rundet auf 1/2 bis 1/8

(define (round/x-helper m)
  (lambda (n) (round4 (/ (round (* n m)) m))))

(define round/2 (round/x-helper 2))
(define round/3 (round/x-helper 3))
(define round/4 (round/x-helper 4))
(define round/5 (round/x-helper 5))
(define round/6 (round/x-helper 6))
(define round/7 (round/x-helper 7))
(define round/8 (round/x-helper 8))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; quality - gewichteter zufall

(define (quality listenliste anzahl)
  (let ((sum (apply + (map cadr listenliste))))
    (letrec ((helper (lambda (ls ind result rest)
		       (if (<= rest 0)
			   result
			   (let ((weight (cadar ls)))
			     (if (<= ind weight)
				 (helper listenliste (random sum) (cons (caar ls) result) (- rest 1))
				 (helper (cdr ls) (- ind weight) result rest)))))))
      (helper listenliste (random sum) '() anzahl))))

;; (quality '((1 1) (2 10) (3 100)) 10)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; pick

(define (pick l)
  (list-ref l (random (length l))))

;; (pick '(1 2 3 4))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; pick-n

(define (pick-n liste n)
  (letrec ((h (lambda (z res)
		(if (= z n) res (h (+ z 1) (cons (pick liste) res))))))
    (h 0 '())))

;; (pick-n '(1 2 3 4) 7)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; index mit mehreren boxen
;; (multi-index (liste1 liste2 liste3...) indexliste)

(define (multi-index listen indexliste)
  (map (lambda (x) (pick (list-ref listen x))) indexliste))

;; (multi-index '((1 2 3) (10 20)) '(0 0 0 1 0))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; mix-pick

(define (mix-pick listen anzahl-liste)
  (letrec ((h (lambda (l anz res)
		(if (null? anz)
		    res
		    (h (rotate l) (cdr anz) (append res (pick-n (car l) (car anz))))))))
    (h listen anzahl-liste '())))

;; (mix-pick '((1 2 3) (11 22 33)) '(1 2 3 4))    -> 1 aus liste1, 2 aus liste2, 3 aus liste1, 4 aus liste2

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; list-ref bis 20
;; und
;; last element

(define (1st-20th-helper n)
  (lambda (liste) (list-ref liste (- n 1))))

(define 1st (1st-20th-helper 1))
(define 2nd (1st-20th-helper 2))
(define 3rd (1st-20th-helper 3))
(define 4th (1st-20th-helper 4))
(define 5th (1st-20th-helper 5))
(define 6th (1st-20th-helper 6)) 
(define 7th (1st-20th-helper 7))
(define 8th (1st-20th-helper 8))
(define 9th (1st-20th-helper 9))
(define 10th (1st-20th-helper 10))
(define 11th (1st-20th-helper 11))
(define 12th (1st-20th-helper 12))
(define 13th (1st-20th-helper 13))
(define 14th (1st-20th-helper 14))
(define 15th (1st-20th-helper 15))
(define 16th (1st-20th-helper 16))
(define 17th (1st-20th-helper 17))
(define 18th (1st-20th-helper 18))
(define 19th (1st-20th-helper 19))
(define 20th (1st-20th-helper 20))

(define (last liste)
  (car (reverse liste)))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; tonhoehen-mapping

(define (ton-map-helper proc1 proc2 faktor)
  (lambda (tiefster hoechster liste)
    (letrec ((h (lambda (l res)
		  (cond ((null? l)
			 res)
			((and (>= (car l) tiefster)
			      (<= (car l) hoechster))
			 (h (cdr l) (append res (list (car l)))))
			((> (car l) hoechster)
			 (h (append (list (proc1 (car l) (* faktor (+ (random 2) 1)))) (cdr l)) res))
			((< (car l) tiefster)
			 (h (append (list (proc2 (car l) (* faktor (+ (random 2) 1)))) (cdr l)) res))))))
      (h liste '()))))

;; midi tonhoehen-mapping

(define midi-ton-map (ton-map-helper - + 12.))

;; (midi-ton-map 60 72 '(24 36 62 80))

;; frequenz tonhoehen-mapping

(define freq-ton-map (ton-map-helper / * 2.))

;; (freq-ton-map 100 200 '(100 200 300 400))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; zaehlt die haeufigkeit eines elementes in einer liste

(define (howmuch? el liste)
  (letrec ((h (lambda (l z)
		(cond ((null? l) z)
		      ((number? el)
		       (if (= (car l) el)
			   (h (cdr l) (+ z 1))
			   (h (cdr l) z)))
		      (else (if (equal? (car l) el)
				(h (cdr l) (+ z 1))
				(h (cdr l) z)))))))
    (h liste 0)))

;; (howmuch? 1 '(1 3 45 1.0))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; rotiert eine liste

(define (rotate liste)
  (append (cdr liste) (list (car liste))))

;; (rotate '(1 2 3))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; wiederholungen eines elements werden entfernt

(define (single-elements liste)
  (letrec ((h (lambda (l res)
		(cond ((null? l)
		       res)
		      ((member? (car l) res)
		       (h (cdr l) res))
		      (else (h (cdr l) (append res (list (car l)))))))))
    (h liste (list (car liste)))))

;; (single-elements (rep-list '(1 2 3) 2))
;; (single-elements (rep-list '(1 2 3 3.0) 2))
;; (single-elements (rep-list '(a b c) 2))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; repeat

(define (rep element n)
  (letrec ((h (lambda (z res)
		(cond ((= z n)
		       res)
		      (else (h (+ z 1) (cons element res)))))))
    (h 0 '())))

;; (rep 2 5)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; haengt listen und nicht-listen aneinander

(define (append-all . elements)
  (let ((list (letrec ((h (lambda (l res)
			    (cond ((null? l)
				   res)
				  ((pair? (car l))
				   (h (cdr l) (append res (car l))))
				  (else (h (cdr l) (append res (list (car l)))))))))
		(h elements '()))))
    list))

;; (append-all 1 '(2 3) "a" '("b") 'c (sort (rand 1 2 3)))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; darstellung einer zahl als binaer-code

(define (bin n)
  (letrec ((h (lambda (res b n)
		(cond ((< b 1) res)
		      ((and (null? res) (>= n (* b 2))) (h res (* b 2) n))
		      (else (if (>= n b)
				(h (append res '(1)) (/ b 2) (- n b))
				(h (append res '(0)) (/ b 2) n)))))))
    (h '() 1 n)))

;; (bin 13)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; darstellung eines binaer-codes als zahl

(define (de-bin l)
  (let ((show (lambda a (for-each display a) (newline))))
    (letrec ((h (lambda (l res n)
		  (if (null? l)
		      res
		      (if (> (car l) 1)
			  (error "falscher binaer-code!")
			  (h (cdr l) (+ res (* (car l) n)) (* n 2)))))))
      (h (reverse l) 0 1))))

;; (de-bin '(1 1 0 1))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; quersumme

(define (quersumme n)
  (letrec ((h (lambda (s res)
		(cond ((= (string-length s) 0)
		       res)
		      ((not (string->number (substring s 0 1)))
		       (h (substring s 1 (string-length s)) res))
		      (else (h (substring s 1 (string-length s)) (+ res (string->number (substring s 0 1)))))))))
    (h (number->string n) 0)))

;; (quersumme 123)
;; (quersumme 123.0001)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; rekursive quersumme

(define (rek-quersumme n)
  (letrec ((h (lambda (m)
		(if (> m 9)
		    (h (quersumme m))
		    (quersumme m)))))
    (h n)))

;; (rek-quersumme 123.0004)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; fraktal

(define (fraktal liste proc generationen)
  (let* ((verhaeltnis (let ((a (exact->inexact (car liste))))
			(letrec ((h (lambda (l res)
				      (if (null? l)
					  res
					  (h (cdr l) (append res (list (/ (car l) a))))))))
			  (h (cdr liste) '(1.0)))))
	 (list-map (lambda (l1 l2)
		     (letrec ((h (lambda (l res)
				   (if (null? l) res
				       (h (cdr l) (append res (list (map (lambda (x) (* x (car l))) l2))))))))
		       (h l1 '())))))
    (letrec ((h (lambda (z temp-res res)
		  (if (> z generationen)
		      (list-map verhaeltnis res)
		      (h (+ z 1) (map proc (apply append (list-map temp-res verhaeltnis))) (append res temp-res))))))
      (h 0 (list (car liste)) '()))))

;; (fraktal '(1 2 3) (lambda (x) (/ x 10)) 2)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; fakultaet

(define (fakultaet n)
  (letrec ((h (lambda (z res)
		(if (> z n) res (h (+ z 1) (* res z))))))
    (h 1 1)))

;; (fakultaet 5)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; funktion (f (x))

(define (function x1 x2 anzahl funct)
  (map funct (lini x1 x2 anzahl)))

;; (function 0 10 11 (lambda (x) (expt x 2)))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; maximum

(define (maxi liste)
  (letrec ((h (lambda (l res)
		(cond ((null? l) res)
		      ((> (car l) res) (h (cdr l) (car l)))
		      (else (h (cdr l) res))))))
    (h liste (car liste))))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; minimum

(define (mini liste)
  (letrec ((h (lambda (l res)
		(cond ((null? l) res)
		      ((< (car l) res) (h (cdr l) (car l)))
		      (else (h (cdr l) res))))))
    (h liste (car liste))))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; mapping von zahlen in einen anderen bereich

(define (mapping liste grenze-unten grenze-oben)
  (let* ((lowest (mini liste))
	 (x (/ (- (maxi liste) lowest) 2))
	 (y (/ (- grenze-oben grenze-unten) 2.)))
    (map (lambda (n) (round3 (+ grenze-unten (* (/ (- n lowest) x) y)))) liste)))

;; (mapping '(1 2 3) 15 25)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; exponentielle interpolation

(define (expi start ende anzahl exponent)
  (if (<= anzahl 1)
      (error "anzahl zu klein - minimum ist 2"))
  (mapping (function 0 1 anzahl (lambda (x) (expt x exponent))) start ende))

;; (expi 10 20 11 1)
;; (expi 10 20 11 4)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; permutationen mit wiederholungssperre

(define (perm liste anzahl wiederholungssperre)
  (letrec ((h (lambda (l w-z a-z res)
		(cond ((= a-z anzahl)
		       (reverse res))
		      ((= w-z wiederholungssperre)
		       (h liste 0 a-z res))
		      (else (let ((i (random (length l))))
			      (h (replace l i) (+ w-z 1) (+ a-z 1) (cons (list-ref l i) res))))))))
    (h liste 0 0 '())))

;; (perm '(1 2 3 4 5) 10 5)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; from-to

(define (from-to from to anzahl ueberlappung)
  (let ((change-first-and-last (lambda (l) (append (list from) (replace (cdr l) (- (length l) 2) to))))
	(a (if (> from to) to from))
	(b (if (> from to) from to)))
    (change-first-and-last (map round0 (mapping (jitter (lini from to anzahl) ueberlappung) a b)))))

;; (from-to 0 10 11 2)
;; (index '(a b c d e) (from-to 1 3 10 1))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; sinus (phasenlage in grad)

(define (sin-phase phasenlage)
  (round4 (sin (* (/ 3.1415926535 180) phasenlage))))

;; (sin-phase 45)
;; (map sin-phase (dist 0 45 17))
;; (map sin-phase (lini 0 360 100))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; cosinus (phasenlage in grad)

(define (cos-phase phasenlage)
  (sin-phase (+ phasenlage 90)))

;; (cos-phase 45)
;; (map cos-phase (dist 0 45 17))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; jitter

(define (jitter liste n)
  (if (= n 0) liste
      (letrec ((h (lambda (l res)
		    (let ((r (/ (random (* n 20000)) 10000)))
		      (if (null? l) (reverse res) (h (cdr l) (cons (round4 (+ (- (car l) n) r)) res)))))))
	(h liste '()))))
  
(define (jitter* liste n)
  (if (= n 0) (error "n muss > 0")
      (letrec ((h (lambda (l res)
		    (let ((r (expt n (random 2.0))))
		      (if (null? l) (reverse res) (h (cdr l) (cons (round4 (* (/ (car l) n) r)) res)))))))
	(h liste '()))))

;; (jitter '(1 2 3) 1)
;; (jitter* '(1 2 3) 2)

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;; shake

(define (shake liste)
  (letrec ((h (lambda (l res)
		(if (null? l) res
		    (let ((r (random (length l))))
		      (h (replace l r) (cons (list-ref l r) res)))))))
    (h liste '())))

;; (shake '(1 2 3 ??? "xyz"))

;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
(define concert-pitch 440)

(define (oktave frequenz)
  (midi->oktave (freq->midi frequenz)))
(define (pitchclass frequenz)
  (midi->pitchclass (freq->midi frequenz)))
(define (centabweichung frequenz)
  (midi->centabweichung (freq->midi frequenz)))
(define (midi->oktave midi)
  (round0 (floor (/ (round midi) 12))))
(define (midi->pitchclass midi)
  (modulo (ex (round (- midi (* 12 (midi->oktave midi))))) 12))
(define (midi->centabweichung midi)
  (let* ((note (round midi))
	 (cent (* 100 (- midi note))))
    (if (> cent 50) (round4 (- cent 100)) (round4 cent))))

(define (midi-info midi)
  (list (midi->oktave midi) (midi->pitchclass midi) (midi->centabweichung midi)))

;; (midi-info 69.5)

(define (freq-info frequenz)
  (midi-info (freq->midi frequenz)))

;; (freq-info 440)

(define (freq->csound frequenz)
  (let* ((pitch (freq-info frequenz))
	 (o (+ 3 (car pitch)))
	 (t (/ (cadr pitch) 100.)))
    (+ o t)))

;; (freq->csound 440)

(define (freq->midi frequenz)
  (round4 (+ 69 (* 12 (logg (/ frequenz concert-pitch) 2)))))

;; (freq->midi 440)

(define (freq->midi/4 frequenz)
  (round/2 (+ 69 (* 12 (logg (/ frequenz concert-pitch) 2)))))

;; (freq->midi/4 454)

(define (midi->freq midi)
  (let* ((t (- midi 69)))
    (if (>= t 0)
	(* concert-pitch (expt 2 (/ t 12)))
	(/ concert-pitch (expt 2 (/ (abs t) 12))))))

;; (midi->freq 69)
;; (midi->freq 69.5)

(define (midi->csound midi)
  (+ 3 (midi->oktave midi) (/ (midi->pitchclass midi) 100.)))

;; (midi->csound 60)

(define (csound->freq n)
  (midi->freq (csound->midi n)))

;; (csound->freq 8.09)

(define (csound->midi n)
  (+ (* (- (round n) 3) 12) (round (* (- n (round n)) 100))))

;; (csound->midi 8.09)